<?php

function mlm_ezmlm_idx_info() {
  return (object) array(
    'name'            =>  t('ezmlm-idx'),
    'supported_types' => array('announcement', 'discussion'),
  );
}

function mlm_ezmlm_idx_create_form($node) {
  $form = array();
 
  $form['mlm_ezmlm_idx_address'] = array(
    '#type'         => 'textfield',
    '#title'        => t('E-mail Address'),
    '#default_value'=> $node->mlm_ezmlm_idx->address,
    '#description'  => t(''),
    '#required'     => TRUE,
  );
  $form['mlm_ezmlm_idx_dsn'] = array(
    '#type'         => 'textfield',
    '#title'        => t('Connection Name'),
    '#default_value'=> $node->mlm_ezmlm_idx->dsn,
    '#description'  => t('If this list is stored in a separate database, <a href="http://drupal.org/node/18429" target="_blank">add the connection string to your settings file</a> and specify the connection name here'),
  );
  $form['mlm_ezmlm_idx_troot'] = array(
    '#type'         => 'textfield',
    '#title'        => t('Table Prefix'),
    '#default_value'=> $node->mlm_ezmlm_idx->troot,
    '#description'  => t(''),
  );
  $form['mlm_ezmlm_idx_autopost'] = array(
    '#type'         => 'checkbox',
    '#title'        => t('Enable Automatic Posting'),
    '#default_value'=> $node->mlm_ezmlm_idx->autopost,
    '#description'  => t('Allow posts to go through automatically, bypassing moderation and list controls. This is not a standard ezmlm feature and requires additional configuration.'),
  );
  $form['mlm_ezmlm_idx_key'] = array(
    '#type'         => 'textfield',
    '#title'        => t('List Key'),
    '#default_value'=> $node->mlm_ezmlm_idx->key,
    '#description'  => t('Required if using automatic posting.  Otherwise, you can safely ignore this'),
  );
  return $form; 
}

function mlm_ezmlm_idx_create_form_validate($node) {
  if (!valid_email_address($node->mlm_ezmlm_idx_address)) {
    form_set_error('mlm_ezmlm_idx_address', t('Invalid mail address format'));
  }
  if (!preg_match('/^[a-z0-9_]*$/i', $node->mlm_ezmlm_idx_troot)) {
    form_set_error('mlm_ezmlm_idx_troot', t('Invalid format for table prefix'));
  }
  if ($node->mlm_ezmlm_idx_autopost) {
    if (!$node->mlm_ezmlm_idx_key) {
      form_set_error('mlm_ezmlm_idx_key', t('List Key is required for auto-moderation'));
    }
  }
}

function mlm_ezmlm_idx_create() {
  return TRUE;
}

function mlm_ezmlm_idx_subscribe($list, $mail, $type='subscriber') {
  db_set_active($list->mlm_ezmlm_idx->dsn);
  $table =  _ezmlm_table($list, $type);
   
  if (db_result(db_query("SELECT 1 FROM %s WHERE address = '%s'", $table, $mail))) {
    return TRUE;                 // already subscribed
  }
  
  db_query("INSERT INTO %s ( hash, address ) 
  		VALUES ( '%s', '%s' )", $table, ezmlm_hash($mail), $mail);
      
  if ($type = 'subscriber') {
    db_query("INSERT INTO %s ( tai, address, fromline, edir) VALUES ( '%s', '%s', '%s', '+')"
      , _ezmlm_table($list, 'slog'), date('Y-m-d H:i:s', time()), $mail, $mail);
  }
  db_set_active();
  return TRUE;
}

function mlm_ezmlm_idx_unsubscribe($list, $mail, $type='subscriber') {
  db_set_active($list->mlm_ezmlm_idx->dsn);
  
  db_query("DELETE FROM %s WHERE address = '%s'", _ezmlm_table($list, $type), $mail);
  if (db_affected_rows() && $type = 'subscriber') {
    db_query("INSERT INTO %s ( tai, address, fromline, edir) VALUES ( '%s', '%s', '%s', '-')"
      , _ezmlm_table($list, 'slog'), date('Y-m-d H:i:s', time()), $mail, $mail);
  }
  db_set_active();
  return TRUE;
}

function mlm_ezmlm_idx_count($list, $type='subscriber') {
  db_set_active($list->mlm_ezmlm_idx->dsn);
  $count = db_result(db_query("SELECT COUNT(1) FROM %s", _ezmlm_table($list,$type)));
  db_set_active();
  return $count;
}

function mlm_ezmlm_idx_subscribers($list, $type='subscriber', $limit=100, $filter='') {
  db_set_active($list->mlm_ezmlm_idx->dsn);
  // it would be cool to join against the users table, but it may be a different DSN
  $res = pager_query("SELECT l.address AS mail FROM ". _ezmlm_table($list, $type)." l 
  		ORDER BY l.address", $limit);
  db_set_active();
  return $res;
}

function mlm_ezmlm_idx_is_subscribed($list, $mail, $type='subscriber') {
  db_set_active($list->mlm_ezmlm_idx->dsn);
  $res = (boolean) db_result(db_query("SELECT 1 FROM %s 
  		WHERE address = '%s' LIMIT 1", _ezmlm_table($list,$type), $mail));
  db_set_active();
  return $res;
}

function mlm_ezmlm_idx_post($list, $mail, $subject, $body) {
  global $user;
  if (is_object($mail)) $mail = $mail->mail;  // you can use an address or a $user object
  
  if (!mlm_is_subscribed($list, $mail) && !mlm_is_subscribed($list, $mail, 'allow')) {
    mlm_subscribe($list, $mail, 'allow');
  }
  
  $address = $list->mlm_ezmlm_idx->address;
  if ($list->mlm_ezmlm_idx->autopost) {
    global $base_url;
    $table   = _ezmlm_table($list, 'post');
    $key     = $list->mlm_ezmlm_idx->key;
    $created = time();
    $id      = db_next_id("{$table}_id");
    $hash    = sha1($id.$subject.$created.$key);
    $address = str_replace('@', "-autopost-{$hash}@", $address);

    db_set_active($list->mlm_ezmlm_idx->dsn);
    db_query("INSERT INTO %s (id, uid, website, subject, created, sent)
      VALUES (%d, %d, '%s', '%s', %d, 0)", 
      $table, $id, $user->uid, $base_url, $subject, $created);
    db_set_active();
  }
  if (mimemail($mail, $address, $subject, $body)) {
    drupal_set_message(t('post sent to %address', array('%address' => $address)));
    return true;
  }
  else {
    drupal_set_message(t('error posting to %address', array('%address' => $address)), 'error');
    return false;
  }
}

function mlm_ezmlm_idx_save($list) {
  if ($list->mlm_ezmlm_idx->autopost) {
    $table   = _ezmlm_table($list, 'post');

    db_set_active($list->mlm_ezmlm_idx->dsn);
    @db_query("SELECT 1 FROM %s", $table);
    if (db_error()) { // table does not exist
      $sql = "CREATE TABLE $table ( id INT, website VARCHAR(100), uid INT, subject VARCHAR(200), created INT, sent INT, PRIMARY KEY (id) )";
      @db_query($sql);
      if (db_error()) { // probably don't have permission
        drupal_set_message(t("You need to create a table to track Drupal posts for auto-moderation.  Execute the following query in the database containing your Ezmlm-idx list tables:")."<br /><br /> <b>$sql;</b><br /><br />");
      }
    }
    db_set_active();
  }
}

function _ezmlm_table($list, $type) {
  $troot = $list->mlm_ezmlm_idx->troot;
  switch ( $type ) {
    
    case 'subscriber':
      return $troot;
    
    case 'moderator':
      return $troot.'_mod';
    
    case 'allow':
      return $troot.'_allow';
    
    case 'deny':
      return $troot.'_deny';
      
    case 'slog':
      return $troot.'_slog';

    case 'post':
      return $troot.'_post';
      
    default:
      return $troot;
      return false;
  }
}
