<?php // $Id: mlm.inc,v 1.7 2006/07/26 06:01:18 vauxia Exp $

/**
 * Obtain a list of backends by searching the backends directory
 */
function _mlm_backends($active_only=true) {
  static $backends = array();
  $active = variable_get('mlm_backends',array());
  if (!count($backends) || $active_only==false) {
    foreach (file_scan_directory(dirname(__FILE__).'/backend', '.inc$') as $file) {
      $backend = $file->name;
      if ($active_only && !in_array($backend, $active)) continue;
      
      if ($info = mlm_backend_call($backend, 'info')) {
        $info->file = $file->filename;
        
        $backends[$backend] = $info;
      }
    }
  }
  return $backends;
}

function _mlm_types($backend='') {
  if($backend) {
    if (!is_object($backend)) {
      $backends = _mlm_backends();
      $backend = $backends[$backend];
    }
    $filtered = array();
    foreach($backend->supported_types as $t) {
      $filtered[$t] = t($t);
    }
    return $filtered;
  }
  
  static $types = array();
  if (!count($types)) {
    foreach (_mlm_backends() as $backend) {
      $temp = array_merge($types, $backend->supported_types);
    }
    sort($types);
    foreach($temp as $type) {
      $types[$type] = t($type);
    }
  }
  return $types;
}

function _mlm_user_form($user) {
  $form = array();
  foreach (mlm_lists(null, null, $user) as $list) {
    if (!isset($form['mlm_register'])) {
      $form['mlm_register'] = array(
        '#type' => 'fieldset', 
        '#title' => t('Mailing Lists'),
        '#collapsible' => TRUE,
        '#collapsed' => FALSE,
        '#weight' => 5,
      );
      $form['mlm_register']['mlm_lists'] = array( '#type' => 'checkboxes' );
    }
    $form['mlm_register']['mlm_lists'][$list->nid] = array(
    		'#type' => 'checkbox',
    	  '#title' => $list->title,
	      '#default_value' => (int) $list->subscribed,
      '#prefix' => '<div class="mlm-list">',
      '#suffix' => '</div>',
    );
  }
  return $form;
}

function _mlm_node_form(&$node) {
  $form = array();
  $backends = array();
  foreach ($mlm_backends = _mlm_backends() as $name => $info) {
    $backends[$name] = $info->name;
  }
  
  if (count($backends) == 0)  {
    drupal_set_message(t('No available list backends'), 'error');
    echo theme('page', t('You must visit the <a href="admin/mlm/backends">backends configuration page</a> and enable one or more backends before you can create a list'));
    exit;
  }
  
  $backend = false;
  if (isset($node->mlm_backend)) $backend = $node->mlm_backend;
  elseif (count($backends) == 1) $backend = key($backends);
  elseif ($backend = $_POST['edit']['mlm_backend']) {
    if (!isset($backends[$backend])) $backend = false;
  }
  
  // create new list form group
  $form['title'] = array(
    '#type'         => 'textfield',
    '#required'     => TRUE,
    '#title'        => t('List Name'),
    '#default_value'=> $node->title,
    '#weight'       => -10,
  );
  if ($backend) {
    $form['mlm_backend'] = array(
      '#type'         => 'hidden',
      '#value'        => $backend,
    );
    $form["mlm_".$backend] = array(
      '#type'         => 'fieldset',
      '#title'        => t('%backend settings', array('%backend' => $backends[$backend])),
      '#weight'       => -6,
    );
    _mlm_node_backend_settings($form["mlm_".$backend], $node, $backend);
    if (isset($node->multipart)) {
      $form['multipart'] = array('#type'=>'value', '#value' => 2);
    }
  }
  else {
    $form['mlm_backend'] = array(
      '#type'         => 'select',
      '#title'        => t('Backend'),
      '#options'      => $backends,
      '#default_value'=> $node->backend,
      '#weight'       => -8,
    );
    $form['multipart'] = array('#type'=>'value', '#value' => 1);
  }
    
  $form['body'] = array(
    '#type'         => 'textarea',
    '#title'        => t('Description'),
    '#rows'         => 20,
    '#default_value'=> $node->body,
    '#weight'       => 5,
  );
  $form['mlm_text_welcome'] = array(
    '#type'         => 'textarea',
    '#title'        => t('Welcome Text'),
    '#rows'         => 10,
    '#default_value'=> $node->mlm_text->welcome,
    '#weight'       => 7,
  );
  $form['mlm_text_goodbye'] = array(
    '#type'         => 'textarea',
    '#title'        => t('Goodbye Text'),
    '#rows'         => 10,
    '#default_value'=> $node->mlm_text->goodbye,
    '#weight'       => 9,
  );
  $form['format'] = filter_form($node->format);
  $form['format']['#weight'] = 3;
  $form['submit'] = array(
    '#type'         => 'submit',
    '#value'        => t('Submit'),
  );
  
  return $form;
}

function _mlm_node_backend_settings(&$form, $node, $backend) {
  // list type selector
  $mlm_backends = _mlm_backends();
  $types = _mlm_types($mlm_backends[$backend]);
  if (count($types) == 1) {
    $form['mlm_type'] = array('#type' => 'value', '#value' => key($types));
  }
  else {
    $form['mlm_type'] = array(
      '#type'         => 'select',
      '#title'        => t('List Type'),
      '#options'      => $types,
      '#required'     => true,
      '#default_value'=> $node->mlm_type,
    );
  }
  
  if ($extra = mlm_backend_call($backend, 'create_form', $node)) {
    foreach ($extra as $name => $field) {
      $form[$name] = $field;
    }
  }
}

function _mlm_node_validate(&$node) {
  if (db_result(db_query("SELECT 1 FROM {node} 
      WHERE type='mlm' AND title = '%s' AND nid != %d",$node->title, $node->nid))) {
    
    form_set_error('title', t('List name must be unique'));
  }
  if ($node->mlm_backend) {
    mlm_backend_call($node->mlm_backend, 'create_form_validate', $node);
  }
}

function _mlm_node_save(&$node) {
  $fields = array('mlm_backend', 'mlm_list_type', 'mlm_type');
  if ($node->is_new) {
    db_query("INSERT INTO {mlm} ( nid, backend, list_type) VALUES (%d, '%s', '%s')" 
      ,$node->nid, $node->mlm_backend, $node->mlm_type);
  }
  else {
    db_query("UPDATE {mlm} set list_type = '%s' WHERE nid = %d", $node->mlm_type, $node->nid);
  }
  db_query("DELETE FROM {mlm_setting} WHERE nid = %d", $node->nid);
  foreach ($node as $name => $val) {
    if (substr($name,0, 4) == 'mlm_' && !in_array($name, $fields) && (is_string($val) || is_int($val))) {
      if (!$val) continue;
      $name = substr($name,4);
      if (substr($name,0,strlen($node->mlm_backend)) == $node->mlm_backend) {
        $type = $node->mlm_backend;
      }
      else {
        $type = substr($name, 0, strpos($name,'_'));
      }
      $name = substr($name, strlen($type)+1);
      db_query("INSERT INTO {mlm_setting} ( nid, type, name, value )
          VALUES (%d, '%s', '%s', '%s')", $node->nid, $type, $name, $val);
    }
  }

  mlm_backend_call($node->mlm_backend, 'save', $node);

  // stay in the admin section
  if (substr($_GET['q'],0,5) == 'admin') {
    $_REQUEST['destination'] = 'admin/mlm';
  }
}

function _mlm_node_delete(&$node) {
  db_query("DELETE FROM {mlm} WHERE nid = %d", $node->nid);
  db_query("DELETE FROM {mlm_setting} WHERE nid = %d", $node->nid);
}

function _mlm_admin($op=null, $list=0, $mail='') {
  
  $lists    = mlm_lists();
  if (!$op) {
    $op = mlm_list_count() ? 'overview' : 'form';
  }
  
  if ($list && !is_object($list)) $list = node_load($list);
  
  switch ($op) {
    case 'overview':
      return _mlm_admin_list_overview();
    
    case 'subscribe':
      mlm_subscribe($list, $mail);
      drupal_goto('admin/mlm');
      return;
    
    case 'unsubscribe':
      mlm_unsubscribe($list, $mail);
      drupal_goto('admin/mlm');
      return;
    
    case 'form':
      global $user;
      if ($nid = arg(3)) {
        return node_form($node = node_load($nid));
      }
      return node_form(array('uid' => $user->uid, 'name' => $user->name, 'type' => 'mlm'));
      
    case 'backends':
      return _mlm_admin_backends();
      
    case 'list':
      $destination = drupal_get_destination(); 
      _mlm_admin_file_import($list);  // pick up any in-progress imports
      $form = array();
      $form['import'] = array(
        	'#type' => 'fieldset',
        '#title' => t('Add Subscribers'),
        '#collapsible' => TRUE,
        '#collapsed' =>TRUE,
      );
      $form['import']['list'] = array(
        '#type' => 'value',
        '#value' => $list,
      );
      if (user_access('upload files')) {
        $form['import']['import_file'] = array(
          '#type' => 'file',
          '#title' => t('Import subscribers from a file'),
        );
        $form['#attributes'] = array('enctype' => 'multipart/form-data');
      }
      $form['import']['import_text'] = array(
        '#type' => 'textarea',
        '#rows' => 20,
        '#title' => t('Add subscribers manually'),
      );
      if ($list->mlm_text->welcome) {
        $form['import']['notify'] = array(
          '#type' => 'checkbox',
          '#title' => t('Send Welcome E-mail'),
        );
      }
      $form['import']['submit'] = array('#type' => 'submit', '#value' => t('add subscribers'));
      $output .= drupal_get_form('_mlm_admin_import', $form);
      
      // todo pukes if list is empty
      if ($res = mlm_subscribers($list, 'subscriber', 1)) {
        $fields = array();
        foreach (array_keys(db_fetch_array($res)) as $f) $fields[$f] = $f;
        
        $form = array();
        $form['export'] = array(
          	'#type' => 'fieldset',
          '#title' => t('Export List'),
          '#collapsible' => TRUE,
          '#collapsed' =>TRUE,
        );
        $form['export']['list'] = array(
          '#type' => 'value',
          '#value' => $list,
        );
        if (count($fields) == 1) {
          $form['export']['fields'] = array(
            '#type' => 'value',
            '#value' => $fields,
          );
        } else {
          $form['export']['fields'] = array(
            '#type' => 'checkboxes',
            '#title' => t('Fields to Export'),
            '#options' => $fields,
            '#default_value' => $fields,
          );
        }
        $form['export']['submit'] = array('#type' => 'submit', '#value' => t('export'));
        $output .= drupal_get_form('_mlm_admin_export', $form);
      }
      
      $result = mlm_backend_call($list->mlm_backend, 'subscribers', $list);
      $rows = array();
      while ($row = db_fetch_array($result)) {
        if (!isset($header)) $header = array_merge(array_keys($row), array(('operations')));
        $row[] = l(t('unsubscribe'), "admin/mlm/unsubscribe/".$list->nid.'/'.current($row),'', $destination);
        $rows[] = $row;
      }
      $output .= theme('table', $header, $rows); 
      $output .= theme('pager', NULL, 50, 0); 
      return $output;
  }
  return;
}

function _mlm_admin_export_submit($form_id, $form_values) {
  $list = $form_values['list'];
  
  if ($res = mlm_subscribers($list,'subscriber',0)) {
    header('Content-Type: text/csv');
    header("Content-Disposition: attachment; filename=".urlencode($list->title).'-'.date("Y-m-d").".csv");
    while ($row = db_fetch_object($res)) {
      if (!isset($header)) {
        $header = array();
        foreach ($form_values['fields'] as $field) {
          if ($field) $header[] = strtoupper($field);
        }
        echo join(',', $header)."\n";
      }
      $return = array();
      foreach ($form_values['fields'] as $field) {
        if ($field) $return[] = $row->$field;
      }
      echo join(',', $return)."\n";
    }
    exit;
  }
  drupal_set_message(t('Error fetching subscribers'));
  drupal_goto();
}

function _mlm_admin_import_submit($form_id, $form_values) {
  $list = $form_values['list'];
  if ($text = $form_values['import_text']) {
    $text = preg_replace(array("/\s/ms","/,+/"), ',', $text);
    foreach (explode(',', $text) as $mail) {
      if (valid_email_address($mail)) {
        mlm_subscribe($list, $mail, 'subscriber', $form_values['notify'], false);
      }
      else {
        drupal_set_message(t('%mail is not a valid email address', array('%mail'=>$mail)),'error');
      }
    }
  }
  // NOTE: actual file import is called from the mlm_admin:list case
}
  
function _mlm_admin_file_import($list, $notify = false) {
  $nid = $list->nid;
  if ( !isset($_SESSION['mlm_filename_'.$nid]) && !count($_FILES)) {
    return;
  }
  
  ini_set('auto_detect_line_endings', TRUE);
    
  if (isset($_SESSION['mlm_filename_'.$nid])) {
    $filename = $_SESSION['mlm_filename_'.$nid];
  }
  else {
    $file = $_FILES['edit']['tmp_name']['import_file'];
    $filename = file_directory_path().'/'.$_FILES['edit']['name']['import_file'];
  
    	if (is_uploaded_file($file)) {
     	move_uploaded_file($file, $filename );
    }
    else {
      drupal_set_message(t('Invalid file'), 'error');
      return FALSE;
    }
    $_SESSION['mlm_filename_'.$nid] = $filename;
  }
  
  if ($timelimit = ini_get('max_execution_time')) {
    $timelimit = time() + $timelimit - 5; // 5-second padding on max_execution_time
  }
  
  if (!$fp = fopen($filename, 'r')) {
    drupal_set_message(t('Error opening file'), 'error');
  }
  
  $fpos = (isset($_SESSION['mlm_fpos_'.$nid])) ? $_SESSION['mlm_fpos_'.$nid] : 0;
  $ret = true;
  fseek($fp, $fpos);
  while ($row = fgetcsv($fp, 4096)) {
      
    // only allowing 1-column imports
    if (count($row) == 1) {
      $mail = $row[0];
    }
    else {
      drupal_set_message(t('Please upload a file with 1 address per line'),'error');
    }
    if (valid_email_address($mail)) {
      $ret = mlm_subscribe($list, $mail, 'subscriber', $notify, true) && $ret;
    }
    else {
      drupal_set_message(t('%mail is not a valid email address', array('%mail'=>$mail)),'error');
    }
        
    // time's up!  clean up and try again
    if ($timelimit && (time() >= $timelimit)) {
      $_SESSION['mlm_fpos_'.$nid] = ftell($fp);
      drupal_set_message(t('File too long for one load. Trying to break it up'));
      drupal_goto($_GET['q']);
    }
  }
  if ($ret) {
    drupal_set_message(t('Subscribers imported successfully'));
  }
  unlink($filename);
  unset($_SESSION['mlm_fpos_'.$nid], $_SESSION['mlm_filename_'.$nid]);
  drupal_goto($_GET['q']);
}

function _mlm_admin_list_overview() {
  $backends = _mlm_backends();
  $sql = "SELECT n.nid, n.status, l.list_type, l.backend, title
      FROM {node} n LEFT JOIN {mlm} l USING (nid) 
      WHERE type='mlm'"; 
  $header = array( 
    array('data' => t('Name'), 'field' => 'title', 'sort' => 'asc'), 
    array('data' => t('Active'), 'field' => 'status'), 
    array('data' => t('Backend'), 'field' => 'backend'),
    array('data' => t('Type'), 'field' => 'type'),
    array('data' => t('Subscribers')),
    array('data' => t('Operations'), 'colspan' => '2') 
  ); 
  $sql .= db_rewrite_sql(tablesort_sql($header)); 
  $result = pager_query($sql, 50); 

  $destination = drupal_get_destination(); 
  while ($row = db_fetch_object($result)) { 
    $count = mlm_backend_call($row->backend, 'count',node_load($row->nid));
    if ($count) $count = number_format((int) $count);
    $rows[] = array(
      l($row->title, 'node/'.$row->nid), 
      $row->status, 
      $backends[$row->backend]->name,
      $row->list_type, 
      array('data'=>$count ? l($count, "admin/mlm/list/$row->nid") : '-', 'align'=>'right'),
      l(t('edit'), "admin/mlm/form/$row->nid", array(), $destination).' ',
      '',
    ); 
  } 

  if (!$rows) { 
    $rows[] = array(array('data' => t('No lists available.'), 'colspan' => '4')); 
  } 

  $output = theme('table', $header, $rows); 
  $output .= theme('pager', NULL, 50, 0); 
  return $output;
}

function _mlm_admin_form_submit($form_id, $form_values) {
  $backend = $form_values['backend'];
  if (mlm_backend_call($backend, 'create_form')) {
    
  }
}

function _mlm_admin_backends() {
  $backends = _mlm_backends(false);
  $active = variable_get('mlm_backends', array());
  
  $functions = array(
    'create' => t('create'),
    'delete' => t('delete'),
    'post' => t('post'),
    'is_subscribed' => t('is&nbsp;subscribed'),
    'subscribe' => t('subscribe'),
    'unsubscribe' => t('unsubscribe'),
    'count' => t('&nbsp;count'),
    'subscribers' => t('subscribers'),
  );
  
  $form = array();
  $form['functions'] = array('#type' => 'value', '#value'=> $functions);
  $form['backends'] = array('#type' => 'value', '#value'=> $backends);
  $form['listing'] = array();
  foreach ($backends as $name => $b) {
    $form['listing'][$name] = array();
    $form['listing'][$name]['name'] = array(
      '#type' => 'markup',
      '#value' => '<b>'.$b->name.'</b>',
      '#title' => t('backend'),
    );
    $form['listing'][$name][$name] = array(
      '#type' => 'checkbox',
      '#title' => t('enabled'),
      '#default_value' => (int) in_array($name, $active),
    );
    foreach ($functions as $f => $val) {
      $form['listing'][$name][$f] = array(
        '#type' => 'markup',
        '#value' => function_exists('mlm_'.$name.'_'.$f) ? '<b>X</b>' : '-',
      );
    }
  }
  $form['submit'] = array('#type' => 'submit', '#value'=> t('Save configuration'));
  $out = drupal_get_form('_mlm_backends', $form);
  
  foreach($active as $name) {
    $b = $backends[$name];
    if ($settings = _mlm_admin_backend_settings($name)) {
      $form = array();
      $form[$name] = array(
        '#type' => 'fieldset',
        '#collapsible' => true,
        '#collapsed' => true,
        '#title' => $b->name,
      );
      foreach($settings as $k=>$v) $form[$name][$k] = $v;
      $form[$name]['submit'] = array('#type' => 'submit', '#value'=> t('Save configuration'));
      $out .= drupal_get_form($name.'_mlm_settings', $form, '_mlm_backend');
    }
  }
  return $out;
}

function _mlm_admin_backend_settings($backend) {
  $form = array();
  if ($settings = mlm_backend_call($backend, 'backend_settings')) {
    $form = array_merge($form, $settings);
  }
  return $form;
}

function theme__mlm_backends(&$form) {
  $rows = array();
  
  // generate the header and the 1st row (available functions) from the incoming form
  $headers = array();
  $row = array();
  foreach($first = current($form['listing']) as $field) {
    if($field['#title']) {
      $headers[] = $field['#title'];
      $row[] = '&nbsp';
    }
  }
  $cols = count($first);
  $width = floor(100 / $cols ).'%';
  $functions = $form['functions']['#value'];
  $headers[] = array('data'=> t('avaliable functions'), 'colspan'=> count($functions));
  $row = array_merge($row, $functions);
  
  $rows = array($row);
  foreach ($form['listing'] as $i => $backend) {
    if (substr($i, 0, 1) == '#') continue;
    
    $row = array();
    foreach ($backend as $j => $field) {
      if (substr($j, 0, 1) == '#') continue;
      
      unset($form['listing'][$i][$j]['#title']);
      $row[] = array('data'=>form_render($form['listing'][$i][$j]), 'width'=>$width, 'align'=> count($row) ? 'center' : 'left');
    }
    $rows[] = $row;
  }
  return theme('table', $headers, $rows).form_render($form);
}

function _mlm_backends_submit($form_id, $form_values) {
  $backends = array();
  foreach ($form_values['backends'] as $name => $b) {
    if (isset($form_values[$name]) && $form_values[$name]) $backends[] = $name;
  }
  variable_set('mlm_backends', $backends);
  drupal_set_message(t('The configuration options have been saved.'));
  drupal_goto($_GET['q']);
}

function _mlm_subscriptions() {
  global $user;
  drupal_set_title(t('Subscription preferences'));
  
  // parse out the nid and email from the url - they can be in any order
  foreach (range(1,4) as $arg) {
    $arg = arg($arg);
    if (is_int($arg)) $nid = $arg;
    elseif (valid_email_address($arg)) $mail = $arg;
  }
  
  // set a reasonable $mail default
  if(!$mail && $user) $mail = $user->mail;
  if (!valid_email_address($mail)) $mail = '';
  
  return _mlm_subscriptions_form($user, $mail);
}

function _mlm_subscriptions_form($user=null, $mail='') {
  if (!count($lists = mlm_lists(null, null, $mail))) {
    return t('There are no mailing lists for this site');
  }
  
  $form = array();
  if ($mail && $mail == $user->mail) {
    $form['mlm_mail'] = array (
  		  '#type' => 'value',
  		  '#value' => $mail,
    );
  }
  else {
    $form['mlm_mail'] = array (
  		  '#type' => 'textfield',
      '#title' => t('E-Mail Address'),
      '#default_value' => $mail ? $mail : t('email address'),
      '#size' => 20,
      '#maxlength' => 80 
    );
    if (!$mail) {
      $form['mlm_mail']['#attributes'] = array('onclick' => "this.value='';");
    }
  }
  $form['mlm_lists'] = array(
  	  '#type' => 'checkboxes',
    '#title' => t('Mailing Lists'),
  );
  foreach ($lists as $l) {
    $form['mlm_lists'][$l->nid] = array(
    	  '#type' => 'checkbox',
    	  '#default_value' => (int) $l->subscribed,
    	  '#title' => l($l->title, 'node/'.$l->nid),
      '#prefix' => '<div class="mlm-list">',
      '#suffix' => '</div>',
    	  '#description' => $l->teaser,
    );
  }

  $form['submit'] = array('#type' => 'submit', '#value' => t('Save Preferences'));
  return drupal_get_form('mlm_form_subscriptions', $form, 'mlm_form');  
}

function _mlm_save_subscriptions($mail, $selections = array(), $type='subscriber', $notify=true) {
  $lists = mlm_lists(null, null, $mail);
  
  foreach ($selections as $nid => $val) {
    if (!$val && $lists[$nid]->subscribed) {
      mlm_unsubscribe($lists[$nid], $mail, $type, $notify);
    }
    elseif ($val && !$lists[$nid]->subscribed) {
      mlm_subscribe($lists[$nid], $mail, $type, $notify);
    }
  }
}

function _mlm_subscriber_block_configure($edit=array()) {
  if (count($edit))  {
    foreach ($edit['mlm_block_lists'] as $k=>$v) {
      if (!$v) unset($edit['mlm_block_lists'][$k]);
    }
    variable_set('mlm_block_title', $edit['mlm_block_title']);
    variable_set('mlm_block_helptext', $edit['mlm_block_helptext']);
    variable_set('mlm_block_lists', $edit['mlm_block_lists']);
    variable_set('mlm_block_hide', $edit['mlm_block_hide']);
    return;
  }

  $options = array();
  $values = variable_get('mlm_block_lists', array());
      
  foreach (mlm_lists() as $list=>$info) {
    $options[$list] = $info->title;
  }
  $form['mlm_block_title'] = array(
    '#type' => 'textfield', 
    '#title' => t('Block Title'), 
    '#default_value' => variable_get('mlm_block_title', t('Subscribe to list')),
  );
  $form['mlm_block_helptext'] = array(
    '#type' => 'textarea', 
    '#title' => t('Help Text '), 
    '#rows' => 5,
    '#default_value' => variable_get('mlm_block_helptext', ''),
  );
  $form['mlm_block_lists'] = array(
    '#type' => 'checkboxes', 
    '#title' => t('Available Lists'),
    '#options' => $options, 
    '#default_value' => $values,
  );
  $form['mlm_block_hide'] = array(
    '#type' => 'checkbox', 
    '#title' => t('Don\'t show subscribed'),
    '#description' => t('Hide available lists from users who are already subscribed'),
    '#default_value' => variable_get('mlm_block_hide', 1),
  );
  return $form;
}
